<template>
  <ModalDialog
    ref="updateModal"
    is-page
    @close="$router.push({ name: 'admin.users' })"
  >
    <ModalHeader>
      <ModalTitle>{{ $t("app.admin.users.update.title") }}</ModalTitle>
    </ModalHeader>
    <ModalForm
      :is-loading="userFetch.isFetching"
      @submit="submit"
    >
      <template #default>
        <AlertError v-if="form.errors && form.errors?.unknown">
          {{ form.errors.unknown }}
        </AlertError>
        <template v-if="userFetch.hasErrors">
          <ErrorState />
        </template>
        <template v-else>
          <FormInput
            id="name"
            :errors="form.errors.name"
          >
            <FormLabel>{{ $t("app.admin.users.form.name") }}</FormLabel>
            <FormText
              v-model="form.values.name"
              autocomplete="off"
              name="name"
              required
            />
          </FormInput>
          <FormInput
            id="email"
            :errors="form.errors.email"
          >
            <FormLabel>{{ $t("app.admin.users.form.email") }}</FormLabel>
            <FormText
              v-model="form.values.email"
              autocomplete="off"
              name="email"
              required
              type="email"
            />
          </FormInput>
          <FormInput
            id="user_role_id"
            :errors="form.errors.user_role_id"
          >
            <FormLabel>{{ $t("app.admin.users.form.user_role_id") }}</FormLabel>
            <UserRoleSelect
              v-model="form.values.user_role_id"
              required
            />
          </FormInput>
        </template>
      </template>
      <template #footer>
        <FormSubmit
          :processing="form.isFetching"
          class="btn btn-black"
        >
          {{ $t("app.admin.users.update.submit") }}
        </FormSubmit>
        <UserDeleteModal
          v-if="hasPermission(Permission.ADMIN_USERS_DELETE)"
          :id="String($route.params.id)"
          v-slot="{ open }"
          @confirm="onDeleted"
        >
          <button
            class="btn btn-outline-red"
            type="button"
            @click="open"
          >
            <ButtonIcon :icon="TrashCanRegularIcon">
              {{ $t("app.admin.users.delete.title") }}
            </ButtonIcon>
          </button>
        </UserDeleteModal>
      </template>
    </ModalForm>
  </ModalDialog>
</template>

<script lang="ts" setup>
import ModalHeader from "@/js/contexts/shared/ui/components/modal/ModalHeader.vue";
import ModalDialog from "@/js/contexts/shared/ui/components/modal/ModalDialog.vue";
import ModalTitle from "@/js/contexts/shared/ui/components/modal/ModalTitle.vue";
import ModalForm from "@/js/contexts/shared/ui/components/modal/ModalForm.vue";
import FormSubmit from "@/js/contexts/form/ui/components/FormSubmit.vue";
import useForm from "@/js/contexts/shared/composables/useForm";
import FormText from "@/js/contexts/form/ui/components/FormText.vue";
import FormInput from "@/js/contexts/form/ui/components/FormInput.vue";
import FormLabel from "@/js/contexts/form/ui/components/FormLabel.vue";
import UserRoleSelect from "@/js/contexts/admin/user-roles/ui/components/UserRoleSelect.vue";
import type { UserUpdate } from "@/js/contexts/shared/models/User";
import useFetch from "@/js/contexts/shared/composables/useFetch";
import ErrorState from "@/js/contexts/shared/ui/components/error/ErrorState.vue";
import AlertError from "@/js/contexts/shared/ui/components/alert/AlertError.vue";
import useModal from "@/js/contexts/shared/composables/useModal";
import UserDeleteModal from "@/js/contexts/admin/users/modals/UserDeleteModal.vue";
import { Permission } from "@/js/contexts/shared/models/Permission";
import useAuth from "@/js/contexts/auth/composables/useAuth";
import ButtonIcon from "@/js/contexts/shared/ui/components/button/ButtonIcon.vue";
import TrashCanRegularIcon from "@/svg/icons/trash-can-regular.svg?component";
import useNotification from "@/js/contexts/shared/composables/useNotification";

type Response = { name: string; email: string; user_role_id: number | null };

const emit = defineEmits<{
  fetch: [];
}>();

const { t } = useI18n();
const { successNotification } = useNotification();
const { hasPermission } = useAuth();
const updateModal = ref<InstanceType<typeof ModalDialog>>();
const modal = useModal(updateModal);
const route = useRoute();
const userFetch = useFetch(`/api/admin/users/${route.params.id}`, { isFetching: true });

const form = useForm<UserUpdate>({
  name: "",
  email: "",
  user_role_id: null,
});

onMounted(() => {
  userFetch.get<Response>().then((response) => {
    form.values.name = response.data.name;
    form.values.email = response.data.email;
    form.values.user_role_id = response.data.user_role_id;
    form.updatedValues();
  });
});

const submit = () => {
  form.put(`/api/admin/users/${route.params.id}`, {
    onSuccess: () => {
      emit("fetch");
      modal.close();
      successNotification(t("app.admin.users.update.success"));
    },
  });
};
const onDeleted = () => {
  emit("fetch");
  modal.close();
};
</script>
